@@ -520,7 +520,7 @@ Input(client_data, source, id)
 	    if (gvw->ghostview.ps_input &&
 		gvw->ghostview.ps_input->seek_needed) {
 		if (gvw->ghostview.ps_input->len > 0)
-		    fseek(gvw->ghostview.ps_input->fp,
+		    GV_FSEEK(gvw->ghostview.ps_input->fp,
 			  gvw->ghostview.ps_input->begin, SEEK_SET);
 		gvw->ghostview.ps_input->seek_needed = False;
 		gvw->ghostview.bytes_left = gvw->ghostview.ps_input->len;
@@ -1389,6 +1389,7 @@ StartInterpreter(w)
     char *dptr;
     int argc = 0;
     int ret;
+    void* toBeFreed = 0;
     String filename = 0;
 
     BEGINMESSAGE(StartInterpreter)
@@ -1423,6 +1424,20 @@ StartInterpreter(w)
 	 while (isspace(*dptr)) dptr++;
     }
     argv[argc++] = "-dNOPAUSE";
+    if (gv_pdf_password)
+    {
+       char* password;
+       
+       String parameter = malloc(100);
+       toBeFreed = (void*) parameter;
+       strcpy(parameter, "-sPDFPassword=");
+       password = quote_filename(gv_pdf_password);
+       strcat(parameter, password);
+       GV_XtFree(password);
+       argv[argc++] = parameter;
+    }
+    
+    
     if (gvw->ghostview.quiet) argv[argc++] = "-dQUIET";
     if (gvw->ghostview.safer) 
       {
@@ -1545,6 +1560,8 @@ StartInterpreter(w)
 	perror(buf);
 	_exit(EXIT_STATUS_ERROR);
     } else {
+        if (toBeFreed)
+	   free(toBeFreed);
         if (filename)
 	   free(filename);
 	if (gvw->ghostview.filename == NULL) {
@@ -2076,8 +2093,8 @@ Boolean
 GhostviewSendPS(w, fp, begin, len, close)
     Widget w;
     FILE *fp;
-    long begin;
-    unsigned int len;
+    gv_off_t begin;
+    gv_off_t len;
     Bool close;
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
