@@ -84,6 +84,7 @@
 #include INC_X11(Xos.h)                /* #includes the appropriate <string.h> */
 #include INC_X11(Xfuncs.h)
 
+#include "callbacks.h"
 #include "types.h"
 #include "file.h"
 #include "misc.h"
@@ -91,6 +92,8 @@
 #include "note.h"
 #include "d_memdebug.h"
 extern Media *gv_medias;
+extern String gv_pdf_password;
+extern int gv_infoSkipErrors;
 
 #ifdef BSD4_2
 #define memset(a,b,c) bzero(a,c)
@@ -170,7 +173,7 @@ typedef struct FileDataStruct_ *FileData;
 typedef struct FileDataStruct_ {
    FILE *file;           /* file */
    int   file_desc;      /* file descriptor corresponding to file */
-   int   filepos;        /* file position corresponding to the start of the line */
+   gv_off_t   filepos;       /* file position corresponding to the start of the line */
    char *buf;            /* buffer */
    int   buf_size;       /* size of buffer */
    int   buf_end;        /* last char in buffer given as offset to buf */
@@ -184,10 +187,11 @@ typedef struct FileDataStruct_ {
 static FileData ps_io_init PT((FILE *));
 static void     ps_io_exit PT((FileData));
 static char    *ps_io_fgetchars PT((FileData, int));
-static int      ps_io_fseek PT((FileData, int));
-static int      ps_io_ftell PT((FileData));
 
-static char    *readline PT((FileData, char **, long *, unsigned int *));
+static int      ps_io_fseek PT((FileData, gv_off_t));
+static gv_off_t     ps_io_ftell PT((FileData));
+static char    *readline PT((FileData, char **, gv_off_t *, unsigned int *));
+
 static char    *gettextline PT((char *));
 static char    *ps_gettext PT((char *,char **));
 static int      blank PT((char *));
@@ -351,10 +355,11 @@ psscan(fileP,filename,filename_raw,filename_dscP,cmd_scan_pdf,filename_uncP,cmd_
     char *line;
                            	/* 255 characters + 1 newline + 1 NULL */
     char text[PSLINELENGTH];	/* Temporary storage for text */
-    long position;		/* Position of the current line */
-    long beginsection;		/* Position of the beginning of the section */
+    gv_off_t position;		/* Position of the current line */
+    gv_off_t beginsection;		/* Position of the beginning of the section */
+
     unsigned int line_len; 	/* Length of the current line */
-    unsigned int section_len;	/* Place to accumulate the section length */
+    gv_off_t section_len;	/* Place to accumulate the section length */
     char *next_char;		/* 1st char after text returned by ps_gettext() */
     char *cp;
     Media dmp;
@@ -485,6 +490,8 @@ unc_ok:
       char cmd[512];
       char s[512];
       mode_t old_umask;
+      FILE* tmp_file;
+      String tmp_filename;
 
       filename_dsc=file_getTmpFilename(NULL,filename_raw);
       /*      sprintf(cmd,cmd_scan_pdf,filename,filename_dsc); */
@@ -507,7 +514,18 @@ unc_ok:
 	  strcat(cmd,pdfpos+4);
 	}
       } else {
-	sprintf(cmd,cmd_scan_pdf,quoted_filename,quoted_filename_dsc);
+        char* password = "";
+	if (gv_pdf_password)
+	{
+	   char parameter[100];
+	   strcpy(parameter, " -sPDFPassword=");
+	   password = quote_filename(gv_pdf_password);
+	   strcat(parameter, password);
+	   GV_XtFree(password);
+	   sprintf(cmd,cmd_scan_pdf,quoted_filename,quoted_filename_dsc, parameter);
+	}
+	else   
+	   sprintf(cmd,cmd_scan_pdf,quoted_filename,quoted_filename_dsc, "");
       }
       GV_XtFree(quoted_filename);
       GV_XtFree(quoted_filename_dsc);
@@ -515,12 +533,50 @@ unc_ok:
       old_umask = umask(0077);
       INFMESSAGE(is PDF)
       INFSMESSAGE(scan command,cmd)
+      
+      tmp_filename=file_getTmpFilename(NULL,filename_raw);
+      tmp_file = fopen(tmp_filename, "w");
+      dup2(2,100);
+      close(2); dup2( fileno(tmp_file), 2);
+      
+
       if (system(cmd) || file_fileIsNotUseful(filename_dsc)) {
+        char line[1000];
+	int found;
+	found = 0;
+      
 	INFMESSAGE(scan subprocess failed)
+	close(2);
+	dup2(100,2);
+	close(100);
+	fclose(tmp_file);
+	
+        tmp_file = fopen( tmp_filename, "r" );
+	while ( fgets( line, 999, tmp_file) )
+	{
+	   if (strstr(line,"This file requires a password for access."))
+	      found = 1;
+	   if (strstr(line,"Password did not work."))
+	      found = 1;
+        }
+	fclose(tmp_file);	
+	unlink((char*) tmp_file);
+	
+	if (found)
+	{
+	   cb_askPassword((Widget)NULL, NULL, NULL);
+	   /* do not show error  */
+	   gv_infoSkipErrors = 1;
+	   goto scan_password_required;
+	   /* TODO? but wait for password dialog */
+	}
+        GV_XtFree(tmp_filename);
+	
 scan_exec_failed:
 	sprintf(s,"Execution of\n%s\nfailed.",cmd);
 scan_failed:
 	NotePopupShowMessage(s);
+scan_password_required:
 	if (tmpfile) fclose(tmpfile);
 	unlink(filename_dsc);
 scan_ok:
@@ -529,6 +585,14 @@ scan_ok:
 	ENDMESSAGE(psscan)
         return(retval);
       }
+
+      close(2);
+      dup2(100,2);
+      close(100);
+      fclose(tmp_file);
+      unlink((char*) tmp_filename);
+      GV_XtFree(tmp_filename);
+
       umask (old_umask);
       tmpfile = fopen(filename_dsc, "r");
       if (!tmpfile) goto scan_exec_failed;
@@ -1515,7 +1579,7 @@ static FileData ps_io_init(file)
    rewind(file);
    FD_FILE      = file;
    FD_FILE_DESC = fileno(file);
-   FD_FILEPOS   = ftell(file);
+   FD_FILEPOS   = GV_FTELL(file);
    FD_BUF_SIZE  = (2*LINE_CHUNK_SIZE)+1;
    FD_BUF       = PS_XtMalloc(FD_BUF_SIZE);
    FD_BUF[0]    = '\0';
@@ -1544,11 +1608,11 @@ ps_io_exit(fd)
 static int
 ps_io_fseek(fd,offset)
    FileData fd;
-   int offset;
+   gv_off_t offset;
 {
    int status;
    BEGINMESSAGE(ps_io_fseek)
-   status=fseek(FD_FILE,(long)offset,SEEK_SET);
+   status=GV_FSEEK(FD_FILE,offset,SEEK_SET);
    FD_BUF_END = FD_LINE_BEGIN = FD_LINE_END = FD_LINE_LEN = 0;
    FD_FILEPOS = offset;
    FD_STATUS  = FD_STATUS_OKAY;
@@ -1560,7 +1624,7 @@ ps_io_fseek(fd,offset)
 /* ps_io_ftell */
 /*----------------------------------------------------------*/
 
-static int
+static gv_off_t
 ps_io_ftell(fd)
    FileData fd;
 {
@@ -1605,19 +1669,6 @@ static char * ps_io_fgetchars(fd,num)
    FD_BUF[FD_LINE_END] = FD_LINE_TERMCHAR; /* restoring char previously exchanged against '\0' */
    FD_LINE_BEGIN       = FD_LINE_END;
 
-#if 0
-   {
-      int fp = (int)(ftell(FD_FILE));
-      if (num<0)  { INFMESSAGE(reading line) }
-      else        { INFMESSAGE(reading specified num of chars) }
-      IIMESSAGE(FD_BUF_SIZE,FD_BUF_END)
-      IIMESSAGE(FD_LINE_BEGIN,FD_LINE_END)
-      INFIMESSAGE(unparsed:,FD_BUF_END-FD_LINE_END)
-      IMESSAGE(fp)
-      IIMESSAGE(FD_FILEPOS,fp-(FD_BUF_END-FD_LINE_END))
-   }
-#endif /* 0 */
-
    do {
       if (num<0) { /* reading whole line */
          if (FD_BUF_END-FD_LINE_END) {
@@ -1707,17 +1758,11 @@ static char * ps_io_fgetchars(fd,num)
       case).
       (Tim Adye, adye@v2.rl.ac.uk)
       */
-      FD_FILEPOS         = ftell(FD_FILE);
+      FD_FILEPOS         = GV_FTELL(FD_FILE);
    } else
 #endif /* USE_FTELL_FOR_FILEPOS */
       FD_FILEPOS        += FD_LINE_LEN;
 
-#if 0
-   SMESSAGE(FD_BUF+FD_LINE_BEGIN)
-   IIMESSAGE(FD_LINE_BEGIN,FD_LINE_END)
-   IIMESSAGE(FD_BUF_END,FD_LINE_LEN)
-#endif
-
    ENDMESSAGE(ps_io_fgetchars)
    return(FD_BUF+FD_LINE_BEGIN);
 }
@@ -1737,7 +1782,7 @@ static char * ps_io_fgetchars(fd,num)
 static char * readline (fd, lineP, positionP, line_lenP)
    FileData fd;
    char **lineP;
-   long *positionP;
+   gv_off_t *positionP;
    unsigned int *line_lenP;
 {
    unsigned int nbytes=0;
